<?php

/* Reminder: always indent with 4 spaces (no tabs). */

/**
 * Abstract base command
 */
require_once 'BaseCommand.php';

/**
 * Provides a user-aware command to be used by developers when building their commands.  This
 * command will help greatly with security as it will require the user to be logged in if it 
 * is instructed to do so (which it is by default).
 *
 * @author Tony Bibbs <tony@geeklog.net>
 *
 */
abstract class PHPARCH_BaseCommandUser extends PHPARCH_BaseCommand {
    /**
     * Handle to user object from session
     * @var DvrsUser
     * @access protected
     */
    protected $user = null;
    
    /**
     * Set of privileges the command requires the user to have at least one of.
     * @var array
     * @access private
     */
    private $privilegesToRun = array();
    
    /**
     * Indicates if the user object is require (i.e. user must have logged in)
     * @access private
     * @var boolean
     */
    private $userRequired = true;
    
    /**
     * Constructor
     *
     * Gets the user object from session.  If one wasn't provided and it is required we will send
     * the user to the login page
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * 
     */
    public function __construct()
    {
        $this->user = unserialize($_SESSION['user']);
        
    	if ($this->getUserRequired() AND !is_object($this->user)) {
            // NOTE this makes use of the global forward feature in MVCnPHP
            return 'doForward:login';
        }
        
        if (!$this->authorizedToRun()) {
            throw new Exception('You do not have sufficient privileges to perform this action.  
                Please note that all attempts to illegally perform actions are logged');
        }
    }
    
    /**
     * Gets the user object
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @return DvrsUser DVRS User object
     *
     */
    public function getUser(){
		return $this->user;
	}
	
	/**
	 * Sets the user object
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @param DvrsUser $user DVRS user object
	 *
	 */
	public function setUser($user){
		$this->user = $user;
	}
	
	/**
	 * This will force the children to execute their command logic.  This is called by the
	 * Controller
	 * 
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public 
	 * @return string Name of forward (if any)
	 *
	 */
	public function execute()
    {    	        
        return $this->processCommand();
    }
    
    /**
     * Sets whether or not this command requires a logged in user or not
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access protected
     * @param boolean $switch Whether or not to require the user be logged in
     *
     */
    protected function setUserRequired($switch)
	{
		$this->userRequired = $switch;
	}
	
	/**
	 * Gets whether or not this command requires the user to be logged in or not
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @return boolean Whether or not the user must be logged in to execute this command.
	 *
	 */
	protected function getUserRequired()
	{
		return $this->userRequired;
	}
	
	/**
	 * Allows views to register one or more privileges that would be required in order to even view
	 * a screen. All privileges will be compared in an OR relationship.  If you need more 
	 * specialized handling then you should override this method.
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @param array $privileges Array of privleges
	 *
	 */
	protected function setPrivilegesToRun($privileges = array())
	{
	    $this->privilegesToRun = $privileges;
	}
	
	/**
	 * Provides a slick method for authorizing a user has sufficient privileges to run a command
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access protected
	 * @return boolean True if the user can view the screen, otherwise false
	 *
	 */
	protected function authorizedToRun()
	{
	    if (count($this->privilegesToRun) == 0) return true;
	    
	    foreach ($this->privilegesToRun as $curPriv) {
	        if ($this->user->hasPrivilege($curPriv)) {
	            return true;
	        }
	    }
	    return false;
	}
}

?>