<?php

/* Reminder: always indent with 4 spaces (no tabs). */

/**
* Data Access Object
*/
require_once 'system/DataAccess/DAOInterface.php';

/**
 * This is the data access object.  It is the proxy used by all business logic to interact with the 
 * database.
 *
 * The DAO layer allows a clean seperation between the details of accessing data from the business 
 * logic so that the developers can concentrate on the business solution instead of the data layer.
 *
 * @author Tony Bibbs <tony@geeklog.net>
 * @copyright 2005
 *
 */
class PHPARCH_PropelDAO implements PHPARCH_DAOInterface {
	/**
	* Handle to XML DOM used to parsed the named queries file
	* @var DomDocument
	* @access private
	*/
	private $dom = null;
	
	/**
	 * Full path to the named query file to use
	 * @var string
	 * @access private
	 */
	private $namedQueryFile = null;
	
	/**
	 * Name of the database we are using
	 * @var string
	 * @access private
	 */
	private $databaseName = null;
	
	/**
	 * Path to the Propel domain objects
	 * @var string
	 * @access private
	 */
	private $domainObjectsPath = null;
	
	/**
	 * Array of queries pulled form named query file
	 * @var array
	 * @access private
	 */
	private $queries = null;
	
	/**
	 * Constructor
	 *
	 * Constructor takes an array of configuration options specific to this DAO implementation
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @param array $options Array of configuration options
	 *
	 */
	public function __construct($options)
	{
	    if (!file_exists($options['namedQueryFile'])) {
	        require_once 'PEAR/Exception.php';
	        throw new PEAR_Exception('Invalid Named Query File Given');
	    }
	    $this->namedQueryFile = $options['namedQueryFile'];
	    $this->databaseName = $options['databaseName'];
	    $this->domainObjectsPath = $options['pathDomainObjects'];
	}
	
	/**
	* Provides generic way to get the name of the database from Propel
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access private
	* @return string Database Name
	* @todo Need to consider how this would change if we had more than one data
	* source.
	*
	*/
	private function getDatabaseName()
	{
		return $this->databaseName;
	}
	
	/**
	* Gets a creole connection object
	*
	* Provides generic way for developers to get a handle to a connection
	* object should they need one.  Using this is discouraged as most DB 
	* interaction should happen using this DAO layer with Propel domain objects
	* or by using the support this class gives for named queries
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	* @return object database connection object
	*
	*/
	public function getConnection()
	{
		$this->con = Propel::getConnection($this->getDatabaseName());
		return $this->con;
	}
	
	/**
	* Allows developers to start a database transaction
	*
	* NOTE: This class uses Propel so transactions occur if you are using
	* the native Propel domain objects.  You should only use this if you plan to 
	* do things in addition to or instead of using Propel
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	*
	*/
	public function beginTransaction()
	{
		if (!is_object($this->con)) {
			$this->getConnection();
		}
		
		$this->con->begin();
	}
	
	/**
	* Allows developers to commit a database transaction
	*
	* NOTE: This class uses Propel so transactions occur if you are using
	* the native Propel domain objects.  You should only use this if you plan to 
	* do things in addition to or instead of using Propel
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	*
	*/
	public function commit()
	{
		if (!is_object($this->con)) {
			$this->getConnection();
		}
		
		try {
			$this->con->commit();
		} catch (PropelException $e) {
			$this->con->rollback();
			throw $e;
		}
	}
	
	/**
	 * Rolls a transaction back
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * 
	 */
	public function rollback()
	{	    
	    $this->con->rollback();
	}
	
	/**
	* Saves an propel domain object.
	*
	* Some may wonder why this DAO layer exists when Propel does much of this.
	* The reason is, frankly, that the decision to use Propel may change.  This
	* layer decreases the Propel dependent code that would be in our commands and
	* views.
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	* @param object $objToSave Propel domain object we want to persist to the DB
	* 
	*/
	public function save(&$objToSave)
	{
		$objToSave->save();
	}
	
	/**
	 * Deletes a record (or records) represented by a Propel domain object
	 *
	 * Some may wonder why this DAO layer exists when Propel does much of this.
	 * The reason is, frankly, that the decision to use Propel may change.  This
	 * layer decreases the Propel dependent code that would be in our commands and
	 * views.
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @param object $objToDelete Propel domain object we want to delete from the the DB
	 *
	 */
	public function delete($objToDelete)
	{
	    if (!is_object($objToDelete)) {
            require_once 'PEAR/Exception.php';
            throw new PEAR_Exception('You must provide a valid Propel domain object');
        }
        
		// Should probably log something at some point
		$objToDelete->delete();
	}
	
	/**
	 * Runs a named query that will do a delete.  When possible you should use domain
	 * objects (see delete() above) but this is nice when you want to delete more than
	 * one object at a time
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @param string $queryName Name of delete query to run
	 *
	 */
	public function deleteWithCondition($queryName, $args=null)
	{
	    // Compile queries (if needed)
		$this->compileQueries();

		if (empty($args)) {
		    $args = array();
		}
		
		// Make sure we found the query
        if (empty($this->queries[$queryName])) {
            require_once 'PEAR/Exception.php';
            throw new PEAR_Exception('Query name not found in named query file');
        }
        
        $sql = $this->queries[$queryName]['sql'];
    	$this->getConnection();
    	$prepStmt = $this->con->prepareStatement($sql);     	
    	$prepStmt->executeQuery($args, null);        	
	}
	
	/** 
	 * Retreive the requested object
	 * 
	 * Takes a propel domain object with the Primary Key(s) set and then retreieves the 
	 * object from the database.
	 *
	 * @author Tony Bibbs <tony@geeklog.net
	 * @access public
	 * @param mixed $objToGet Propel domain object 
	 * @return mixed fully populated Propel domain object
	 *
	 */
	public function get($objToGet)
	{
	    // Should proably log something at some point	
		$className = get_class($objToGet) . 'Peer';		
		require_once $this->domainObjectsPath . "$className.php";
		$pkFields = (array) $objToGet->getPrimaryKey();
		$retObj = call_user_func_array(array($className,'retrieveByPK'), $pkFields);
		
		return $retObj;
	}
	
	/**
	* Searches the database and returns Propel domain object(s)
	*
	* This uses the concept of named queries.  All queries will go in an XML file and each query
	* can be one of two types: 1) Propel 2) SQL.  The first one will use the built in criteria support
	* that Propel has.  The second will simply issue raw SQL.  Named queries have many benefits:
	* 1) Prevents any SQL from being put in business or display logic
	* 2) All SQL is in a single place.  This makes maintenance easier (explained further down)
	* 3) Named queries are easily reused
	* 4) Using this XML format allows future modifications that would allow us to create a 
	*    PHPUnit2 test script to regression test each and every query with test criteria data (not
	*    yet implemented).
	*
	* @author Tony Bibbs <tony@geeklog.net>
	* @access public
	* @param string $queryName Name of query to run
	* @param array $args Criteria arguments
	* @param int $page Page to get
	* @param int $recordCount Records per page
	* @return array Array of Propel domain objects
	* @todo implement paging and the user of the recordCount. Note that Propel has a paging class in
	* its /etc directory (Propel_Pager) or something like that.
	*
	*/
	public function find($queryName, $args=null, $page=null, $recordCount=null)
	{
		// Should log something here
		
		// Ensure arguments are in the right format
		if (!empty($args) AND !is_array($args)) {
		    require_once 'PEAR/Exception.php';
		    throw new PEAR_Exception('The find() method on the DAO layer expects all SQL arguments
		        be wrapped up in an array');
		}
		
		// Compile queries (if needed)
		$this->compileQueries();
		
		// Make sure we found the query
        if (empty($this->queries[$queryName])) {
            require_once 'PEAR/Exception.php';
            throw new PEAR_Exception('Query name not found in named query file');
        }
           
    	// This implements queries using raw SQL
    	$propelPeer = $this->queries[$queryName]['propelPeer'];
    	
    	// Ensure we got a valid Propel Peer name
    	if (!strstr($propelPeer,'Peer') and empty($this->queries[$queryName]['columnArray'])) {
    	    require_once 'PEAR/Exception.php';
    	    throw new PEAR_Exception('Invalid Propel Peer name given.  Please double check the named query file');
    	}
    	
    	// Ensure the required model class is already defined.
    	if ($propelPeer AND !class_exists($propelPeer)) {
    	    $classToInclude = str_replace('Peer','',$propelPeer);
    	    require_once 'PEAR/Exception.php';
    	    throw new PEAR_Exception("Commands or views using PropelDAO are responsible for 
    	       including the classes they need.  You forgot to include $classToInclude");
    	}
    	
    	$sql = $this->queries[$queryName]['sql'];
    	$this->getConnection();
    	$prepStmt = $this->con->prepareStatement($sql);
    	
    	if ($propelPeer) {
    	    $result = $prepStmt->executeQuery($args,ResultSet::FETCHMODE_NUM);
        	$parentObj = get_parent_class($propelPeer);
        	return call_user_func_array(array($parentObj,'populateObjects'), array($result));
    	} else {
    	    $columnArray = $this->queries[$queryName]['columnArray'];        	    
    	    if (is_numeric($columnArray) OR is_null(($columnArray))) {
    	       $result = $prepStmt->executeQuery($args,ResultSet::FETCHMODE_NUM);
    	    } else {        	       
    	       $result = $prepStmt->executeQuery($args,ResultSet::FETCHMODE_ASSOC);        	       
    	    }
    	    return $this->toArray($result, $columnArray);
    	}
	}
	
	/**
	 * Performs arbitrary searches
	 *
	 * Unlike the find() method, this method allows for the dynamic assembly of a query's WHERE
	 * and ORDER BY clauses.  This is particularly useful to search pages that may combine any
	 * numbeer of optional or required search criteria.
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @param string $queryName Query to load and add WHERE/ORDER BY clauses to
	 * @param array $strictParams Array of fieldname=>value entries.  Each entry will use the
	 * SQL equal operator.  Thus, firstName=>'Tony' would product "firstName = 'Tony'" in the 
	 * WHERE clause
	 * @param array $looseParams Array of fieldname=>value entries.  Each entry will use the 
	 * SQL like operator. Thus, firstname=>'Tony' would produce "firstName like '%Tony%'" in
	 * the WHERE clause
	 * @param array $orderByParams Array of fieldname=>order to put into the ORDER BY clause.  So,
	 * if you have array('lastName'=>'desc','salary'=>'asc') you will get:
	 * "ORDER BY lastName desc, salary asc"
	 * @param string $logicalOperator Can be one of the following: OR, AND.  It is used as the glue
	 * between the WHERE clause entries.  If you require a mix of both, you are better just getting
	 * the connection object from the DAO and then construct and run the SQL yourself.  Remember, 
	 * the guideline is to remove as much SQL from our commands/views as possible.  But it is just
	 * a guidline
	 * @param int $page Page number to retrieve (NOT YET IMPLEMENTED)
	 * @param int $recordCount Number of records to get (NOT YET IMPLEMENTED)
	 * @return mixed Array of domain objects
	 * @todo Implement paging and limiting
	 *
	 */
	public function search($queryName, $strictParams, $looseParams, $orderByParams = null, 
	   $logicalOperator = 'OR', $page = null, $recordCount = null)
	{
	    $whereArray = array();
	    $orderByArray = array();
	    
	    $args = array();
	    if (is_array($strictParams)) {
    	    foreach ($strictParams as $fieldName=>$valueToUse) {
    	        $whereArray[] = "($fieldName = ?)";
    	        $args[] = $valueToUse;
    	    }
	    }
	    
	    if (is_array($looseParams)) {	        
    	    foreach ($looseParams as $fieldName=>$valueToUse) {
    	        $whereArray[] = "($fieldName like ?)";
    	        $args[] = "%$valueToUse%";
    	    }
	    }
	    	    
	    // Make sure we got something in the WHERE clause
	    if (count($whereArray) == 0) {
	        require_once 'PEAR/Exception.php';
            throw new PEAR_Exception('You did not pass strictParams or looseParms.  One of them
                is required in order to run your search');
	    }
	    $whereClause = ' WHERE ' . implode(" $logicalOperator ", $whereArray);
	    
	    if (is_array($orderByParams)) {
    	    foreach ($orderByParams as $fieldName=>$sortOrder) {
    	        $orderByArray[] = "$fieldName $sortOrder";
    	    }
	    }
	    
	    // Since an order by is optional, only build it if you got something<br>
        $orderByClause = '';
	    if (count($orderByArray) > 0) {
	       $orderByClause = ' ORDER BY ' . implode(", ", $orderByArray);
	    }
	    
	    // Compile queries (if needed)
		$this->compileQueries();
		
		// Make sure we found the query
        if (empty($this->queries[$queryName])) {
            require_once 'PEAR/Exception.php';
            throw new PEAR_Exception('Query name not found in named query file');
        }
           
    	// This implements queries using raw SQL
    	$propelPeer = $this->queries[$queryName]['propelPeer'];
    	
    	// Ensure we got a valid Propel Peer name
    	if (!strstr($propelPeer,'Peer') and empty($this->queries[$queryName]['columnArray'])) {
    	    require_once 'PEAR/Exception.php';
    	    throw new PEAR_Exception('Invalid Propel Peer name given.  Please double check the named query file');
    	}
    	
    	// Ensure the required model class is already defined.
    	if ($propelPeer AND !class_exists($propelPeer)) {
    	    $classToInclude = str_replace('Peer','',$propelPeer);
    	    require_once 'PEAR/Exception.php';
    	    throw new PEAR_Exception("Commands or views using PropelDAO are responsible for 
    	       including the classes they need.  You forgot to include $classToInclude");
    	}
    	
    	// Get query
    	$sql = $this->queries[$queryName]['sql'];
    	
    	// Add WHERE and ORDER BY clauses
    	$sql = $sql . $whereClause . $orderByClause;
    	
    	// Prepare the SQL.  NOTE: we have to use a prepared statement to avoid having to detect
    	// data types.
    	$this->getConnection();
    	$prepStmt = $this->con->prepareStatement($sql);
    	
    	if ($propelPeer) {
    	    $result = $prepStmt->executeQuery($args,ResultSet::FETCHMODE_NUM);
        	$parentObj = get_parent_class($propelPeer);
        	return call_user_func_array(array($parentObj,'populateObjects'), array($result));
    	} else {
    	    $columnArray = $this->queries[$queryName]['columnArray'];        	    
    	    if (is_numeric($columnArray) OR is_null(($columnArray))) {
    	       $result = $prepStmt->executeQuery($args,ResultSet::FETCHMODE_NUM);
    	    } else {        	       
    	       $result = $prepStmt->executeQuery($args,ResultSet::FETCHMODE_ASSOC);        	       
    	    }
    	    return $this->toArray($result, $columnArray);
    	}
	}
	
	/**
	 * In the rare cases we run SQL that returns data that doesn't map to a domain object, this method
	 * can be used to return an array of values for the columnArray attribute given
	 *
	 * @author Tony Bibbs
	 * @access private
	 * @param ResultSet $creoleResultSet Creole ResultSet object
	 * @param mixed $columnToGet Can be a column name or index
	 * @return array Array of values for the given column
	 *
	 */	 
	public function toArray($creoleResultSet, $columnToGet)
	{
	    $retArray = array();
	    while($creoleResultSet->next()) {
          $retArray[] =  $creoleResultSet->getString($columnToGet);
        }
        return $retArray;
	}
	
	/**
	 * Compiles the named query file
	 *
	 * The XML-based named query file is great as it allows us to apply a schema to validate it, 
	 * however, we don't want to incur the cost of accessing the file *and* parsing it for each and
	 * every query made.  To get around this, this method will compile the named query file into a
	 * PHP file that contains an array representation of the XML.  
	 *
	 * NOTE: this method assumes that a directory will not have more than one named query file in
	 * it.  The destination of the compiled file will be in the same directory as the named query
	 * file and it will be called compiledQueries.php
	 *
	 * @author Tony Bibbs <tony@geeklog.net>
	 * @access public
	 * @todo Get rid of one named query file per directory (i.e. name compiled file name unique)
	 * 
	 */
	private function compileQueries()
	{
	    // First determine of compilation is needed now.
	    $needsCompile = false;
		$arrayFileName = dirname($this->namedQueryFile) . '/compiledQueries.php';
        if (file_exists($arrayFileName)) {
            if (filemtime($arrayFileName) < filemtime($this->namedQueryFile)) {
                $needsCompile = true;
            }
        } else {
            $needsCompile = true;
        }
        if (!$needsCompile) {   
            // File is already compiled, bail
            require "$arrayFileName";
            return;
        }    		
	    
        // Parse named query XML and build arrray
	    $this->dom = new DomDocument();
		$this->dom->load($this->namedQueryFile);
		$xpath = new DOMXPath($this->dom);
		$queryList = $xpath->query('/named_queries/query'); 
		$queryArray = array();   
		foreach ($queryList as $curQuery) {
		    $queryName = $curQuery->getAttribute('name');
	        // Handle raw SQL query
	        $sqlList = $xpath->query("/named_queries/query[@name='$queryName']/sql");
	        $sql = $sqlList->item(0);
	        $propelPeer = $sql->getAttribute('propelPeer');
	        $columnArray = $sql->getAttribute('columnArray');
	        $sqlStmt = $sql->textContent;
	        $queryArray[$queryName] = array('propelPeer'=>$propelPeer,
	                                        'columnArray'=>$columnArray,
	                                        'sql'=>$sqlStmt);		         
		}
		
		// 'Compile' the array by putting it into a PHP file
		file_put_contents($arrayFileName, 
            '<?php $this->queries = ' . var_export($queryArray, true) . '; ?>');
        $this->queries = $queryArray;        
	}
}

?>