<?php

/**
* MVCnPHP - BaseView.class.php
*
* This source file is subject to version 2.02 of the PHP license,
* that is bundled with this package in the file LICENSE, and is
* available at through the world-wide-web at
* http://www.php.net/license/2_02.txt.
* If you did not receive a copy of the PHP license and are unable to
* obtain it through the world-wide-web, please send a note to
* license@php.net so we can mail you a copy immediately.
*
* @author Tony Bibbs <tony@geeklog.net>
* @copyright Tony Bibbs 2003
* @package net.geeklog.mvc
* @version $Id: BaseView.php,v 1.4 2005/03/17 05:08:37 tony Exp $
*
*/

/**
 * Defines the interface all views must implement
 */
require_once 'ViewInterface.php';

/**
* Abstract class for views
*
* @author Tony Bibbs <tony@geeklog.net>
* @package net.geeklog.mvc
*
*/
abstract class MVCnPHP_BaseView implements MVCnPHP_ViewInterface {
    /**
    * @access private
    * @var int
    */
    public $outputMethod = MVC_PRINT;
    
    /**
     * Indicates if a view requires it is run under SSL
     * @var boolean
     * @access private
     */
    private $requiresSSL = false;
    
    /**
     * Constructor
     *
     * Checks if SSL is required and if it is but fails the check it will throw an exception
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @throws Exception Throws an exception if the view fails the SSL check
     *
     */
    public function __construct()    
    {
        if ($this->requiresSSL) {
            if (!stristr($_ENV['SERVER_PROTOCOL'],'https')) {
                throw new Exception('View requires SSL');
            }
        }
    }
    
    /**
     * Sets whether or not SSL is required by the view
     * 
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @param boolean $switch Indicator of whether or not SSL is required
     * 
     */
    public function setSSLRequired($switch)
    {
        $this->requiresSSL = $switch;
    }
    
    /**
     * Gets whether or not SSL is required
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @return boolean Whether or note the view requires SSL
     *
     */
    public function getSSLRequired()
    {
        return $this->requiresSSL;
    }
    
    /**
    * Returns if view should be printed
    *
    * View can be printed using echo or returned as a string.
    * This method is a tool used by the controller to determine
    * if it needs to return a string or not
    *
    * @author Tony Bibbs <tony@geeklog.net>
    * @access public
    * @return boolean True if view is printed, otherwise false
    *
    */
    public function printView()
    {
        if ($this->outputMethod == MVC_PRINT) {
            return true;
        } else {
            return false;
        }
    }
    
    /**
    * Sets views output method
    *
    * View can either be printed immediately using PHP's
    * echo command or the view can be returned as a string
    *
    * @author Tony Bibbs <tony@geeklog.net>
    * @access public
    * @param int $method One of two constanst, _PRINT or _STRING
    *
    */
    public function setOutputMethod($method)
    {
        if ($method == MVC_PRINT) {
            $this->outputMethod = MVC_PRINT;
        } else {
            $this->outputMethod = MVC_STRING;
        }
    }
    
    /**
     * This short piece of code converts a request into a Propel model objec that the 
     * developer can work directly with.
     *
     * It is possible to map form fields directly to model objects so that the command that catches
     * the request can begin working with the objects immediately and not have to do the form field
     * to domain object mapping themselves.  
     *
     * NOTE: should one of the setters cause an exception to get thrown, we simply ignore them and 
     * continue processing.  This is important because Propel's date/time setters() expect a 
     * format argument that we aren't giving them so we'll just ignore it and let the caller fo this
     * method try and figure out the problem.
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public
     * @return array Array of objects
     *
     */
    protected function requestToObjects()
	{
		$postVars = array_keys($_REQUEST);
		foreach ($postVars as $curVarName) {
			if (stristr($curVarName, '->')) {
				$tmpArray = explode('->',$curVarName);
				$objArray[$tmpArray[0]][$tmpArray[1]] = $_REQUEST[$curVarName];
			}
		}
		$objNames = array_keys($objArray);
		$objInstances = array();
		foreach ($objNames as $curObj) {
			if (empty($objInstances[$curObj])) {
				$obj = new $curObj();
				$objInstances[$curObj] = $obj;
			} else {
				$obj = $objInstances[$curObj];
			}
			$objAttributes = array_keys($objArray[$curObj]);
			foreach ($objAttributes as $curAttribute) {
				$fnName = 'set' . ucwords($curAttribute);
				try {
				    $obj->$fnName($objArray[$curObj][$curAttribute]);
				} catch (Exception $e) {
				    // Just continue going on from here. Note we do this so that errors we might 
				    // encounter will not stop processing the request string.				    
				}
			}
			$objInstances[$curObj] = $obj;
		}
		return $objInstances;
	}
}

?>