<?php

/**
* MVCnPHP - XMLLoader.class.php
*
* This source file is subject to version 2.02 of the PHP license,
* that is bundled with this package in the file LICENSE, and is
* available at through the world-wide-web at
* http://www.php.net/license/2_02.txt.
* If you did not receive a copy of the PHP license and are unable to
* obtain it through the world-wide-web, please send a note to
* license@php.net so we can mail you a copy immediately.
*
* @author Tony Bibbs <tony@geeklog.net>
* @copyright Tony Bibbs 2003
* @package net.geeklog.mvc
* @version $Id: XMLLoader.php,v 1.4 2005/03/17 05:08:38 tony Exp $
*
*/

/**
* Base Loader class which does most of the work
*/
require_once 'BaseLoader.php';

/**
* This loader creates a mapping from an XML file
*
* @author Tony Bibbs <tony@geeklog.net>
* @package net.geeklog.mvc
*
*/
class MVCnPHP_XMLLoader extends MVCnPHP_BaseLoader {
    /**
     * Handle to an XML DOM
     * @access protected
     * @var array
     */
    protected $dom = null;
    
    /**
     * Holds array representation of XML config file
     * @access protected
     * @var array
     */
    protected $arrayData = null;
    
    const VIEWS = 'view';
        
    const COMMANDS = 'command';
    
    /**
     * Constructor
     *
     * Initializes object properties and, if the XML has been supplied it will parse it immediately 
     * instead of waiting for an explicit call to parseXML
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     * @param string $xmlConfig XML to parse. This option is most likely passed when using the UI
     * that is included with this package.
     *
     */
    public function __construct($xmlConfig = '')
    {        
        $this->dom = new DomDocument();
        if (!empty($xmlConfig)) {
            $this->dom->loadXML($xmlConfig);
        }
    }
    
    /**
    * Gets config data for given object in the form of
    * a mapping object
    *
    * @author Tony Bibbs <tony@geeklog.net>
    * @access public
    * @param string $name Name of object to get mapping for
    * @param array $arrayData Array of configuration data
    * @return object Mapping object
    * @todo should probably scan $this->arrayData for potentially harmful PHP
    *
    */
    public function getMapping($name, $viewsDir, $commandsDir, $baseURL, $arrayData = '')
    {
        if (!file_exists($arrayData) AND !is_array($this->arrayData)) {
            return false;
        }
        
        if (empty($this->arrayData)) {                
            $xmlData = file_get_contents($arrayData); 
            $arrayFileName = dirname($arrayData);
            $arrayFileName = $arrayFileName . '/compiled_mvcConfig.php';
            $needsCompile = false;
            if (file_exists($arrayFileName)) {
                if (filemtime($arrayFileName) < filemtime($arrayData)) {
                    $needsCompile = true;
                }
            } else {
                $needsCompile = true;
            }

            if ($needsCompile) {   
                $this->dom->loadXML($xmlData);
                if (!$this->dom->schemaValidate(dirname(__FILE__) . '/mvcConfig.xsd')) {
                    throw new Exception('MVCnPHP XML configuration file faield XML schema validation');
                }                
        		if (!empty($xmlData)) {
                    $this->xmlToArray();
        		}
        		file_put_contents($arrayFileName, '<?php $this->arrayData = ' . var_export($this->arrayData, true) . '; ?>');
            } else {    
                require_once "$arrayFileName";
            }
        }
        return parent::getMapping($name, $viewsDir, $commandsDir, $baseURL, $this->arrayData);
    }
    
    /**
     * Converts the MVC XML configuration into an array
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access public
     *
     */
    public function xmlToArray() 
    {
        $arrayData = array();
        $arrayData['views'] = $this->parseSection(MVCnPHP_XMLLoader::VIEWS);
        $arrayData['commands'] = $this->parseSection(MVCnPHP_XMLLoader::COMMANDS);
        $xpath = new DOMXPath($this->dom);
        $tagArray = $xpath->query('/mvc_configuration');
        $configTag = $tagArray->item(0);
        $compile = $configTag->getAttribute('compile');
        $ui = $configTag->getAttribute('ui');
        $arrayData['compile'] = $compile;
        $arrayData['ui'] = $ui;
        $this->arrayData = $arrayData;
    }
    
    /**
     * Parses a section of the XML configuration
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access private
     * @param string $section Use one of the two class constants for this value
     * @return array Array representation of the XML for the given section
     *
     */
    private function parseSection($section)
    {
        $xpath = new DOMXPath($this->dom);
        $tagList = $xpath->query("/mvc_configuration/$section");
        $tagArray = array();
        foreach ($tagList as $curTag) {
            $tagID = $curTag->getAttribute('id');
            $tagName = $curTag->getAttribute('name');
            $tagDefault = $curTag->getAttribute('default');
            $queryStr = sprintf("/mvc_configuration/%s[@id='%s']/forward", $section, $tagID);
            $forwardList = $xpath->query($queryStr);
            $forwardArray = array();
            foreach ($forwardList as $curForward) {
                $forwardID = $curForward->getAttribute('id');
                $forwardType = $curForward->getAttribute('type');
                $forwardTarget = $curForward->textContent;
                $forwardArray[$forwardID] = array('target'=>$forwardTarget,'type'=>$forwardType);
            }
            if (strtolower($tagDefault) == 'true') {
                if (count($forwardArray) > 0) {
                    $tagArray[$tagID] = array('name'=>$tagName, 'forwards'=>$forwardArray, 'default'=>1);
                } else {
                    $tagArray[$tagID] = array('name'=>$tagName, 'default'=>1);
                }
            } else {
                if (count($forwardArray) > 0) {
                    $tagArray[$tagID] = array('name'=>$tagName, 'forwards'=>$forwardArray);
                } else {
                    $tagArray[$tagID] = array('name'=>$tagName);
                }
            }
        }
        return $tagArray;
    }
    
    /**
     * Gets the array representation of MVC XML configuration
     *
     * @author Tony Bibbs <tony@geeklog.net>
     * @access pubic
     * @return array Array representation of MVC configuration
     *
     */
    public function getArrayData()
    {
        return $this->arrayData;
    }    
}

?>