<?php

/* Reminder: always indent with 4 spaces (no tabs). */

/**
 * Department of Administrative Services
 * Information Technology Enterprise
 *
 * ITE PHP5 Enterprise Enterprise Authentication and Authorization (A&A) Client
 *
 * This source file is subject to version 2.02 of the PHP license, that is bundled with this 
 * package in the file LICENSE, and is available at through the world-wide-web at
 * http://www.php.net/license/2_02.txt. If you did not receive a copy of the PHP license and are 
 * unable to obtain it through the world-wide-web, please send a note to license@php.net so we can 
 * mail you a copy immediately.
 *
 * @author Tony Bibbs <tony.bibbs@iowa.gov>
 * @copyright 2005 State of Iowa
 * @version $Id:$
 *
 */

/**
 * The A&A service client interface
 */
require_once 'ServiceInterface.php';

/**
 * The A&A service result code constants
 */
require_once 'Constants.php';

/**
 * This file defines the application program interface into ITE's Enterprise A&A Service
 *
 * This class implements communication to the A&A Serivce using raw XML posted over HTTP or HTTPS.
 * The XML for all requests is in the general format of:
 *
 * <AAService>
 *     <someMethodToCall>
 *         <argument1/>
 *         <argument2/>
 *         <argumentN/>
 *     </someMethodToCall>
 * </AAService>
 *
 * Similarly, all responses to the service take the general format of:
 *
 * <AAService>
 *     <Result value="" method="" appId=""></Result>
 *     <returnVal1/>
 *     <returnVal2/>
 *     <returnVal3/>
 * </AAService>
 *
 * For a complete description of the XML required for each method see the ITE document called
 * XML_Definitions.doc.
 * 
 * @package gov.iowa.das.ite.aaclient
 *
 */
class ITE_AAClient_XMLHandler implements ITE_AAClient_ServiceInterface  {
    
    /**
     * Application ID assigned to the application
     * @var string
     * @access private
     */
    private $appId = null;
    
    /**
     * Host name for the A&A Server to talk to
     * @var string
     * @access private
     */
    private $aaServer = null;
    
    /**
     * Port to talk to the A&A Server over
     * @var int
     * @access private
     */
    private $aaPort = null;
    
    /**
     * Indicates whether or not to use SSL while talking to the A&A Service
     * @var boolean
     * @access private
     */
    private $useSSL = null;
    
    /**
     * Path on the server to send requests to
     * @var string
     * @access private
     */
    private $aaPath = null;
    
    /**
     * Holds instance of the native PHP5 XML DOM
     * @var DomDocument
     * @access private
     */
    private $dom = null;     
    
    /**
     * Constructor
     *
     * Prepares for communication to the A&A Service
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public
     * @param string $aaServer Host name of the A&A Server
     * @param int $aaPort Port to talk over on A&A Server
     * @param boolean $useSSL Whether or not we should talk over SSL to the A&A Service
     *
     */
    public function __construct($appId, $aaServer, $aaPath='/entaa', $useSSL=true, $aaPort='')
    {
        // Set needed configuration options for A&A.
        $this->appId = $appId;
        $this->aaServer = $aaServer;
        $this->aaPort = $aaPort;
        $this->useSSL = $useSSL;
        $this->aaPath = $aaPath;
        
        // Create XML DOM used to build requests.
        $this->dom = new DomDocument();
    }
    /**
     * Authenticates a user
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public 
     * @param string $userId User to authenticate
     * @param string $password Password to authenticate user with
     * @param string $providerId ID of provider to use
     *
     */
    public function &authenticate($userId, $password, $providerId = '')
    {
        $serviceNode = $this->dom->createElement('AAService');
        $methodNode = $this->dom->createElement('Authenticate');
        $appNode = $this->dom->createElement($this->appId);
        $methodNode->appendChild($appNode);
        $methodNode = $this->dom->createElement('Authenticate');
        
        // Application ID node
        $tmpNode = $this->dom->createElement('AppId');
        $tmpNode->setAttribute('value',$this->appId);
        $methodNode->appendChild($tmpNode);
        
        // User ID node
        $tmpNode = $this->dom->createElement('UserId');
        $tmpNode->setAttribute('value',$userId);
        $methodNode->appendChild($tmpNode);
        
        // Password node
        $tmpNode = $this->dom->createElement('Password');
        $tmpNode->setAttribute('value', $password);
        $methodNode->appendChild($tmpNode);
        
        if (!empty($providerId)) {
            // Provider ID node.  This is only used when an application supports authenticating to
            // more than one provider.  For most applications this will not be provided
            $tmpNode = $this->dom->createElement('ProviderId');
            $tmpNode->setAttribute('value', $providerId);
            $methodNode->appendChild($tmpNode);
        }
        
        $serviceNode->appendChild($methodNode);
        $this->dom->appendChild($serviceNode);
        
        $this->makeRequest();
        
        $userObj = $this->getUserFromResponse();
        
        $userObj->setHandler($this);
        
        return $userObj;
    }
    
    /**
     * Creates a new Enterprise A&A account in the repository tied to the application
     *
     * NOTE: this method does not work yet on the Active Directory provider.
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access public
     * @param string $userId ID to assign to the new user
     * @param string $password Initial password for the user
     * @param string $fname User's first name
     * @param string $lname User's last name
     * @param string $email User's email address
     * @param array $privArray Array of privileges to assign to the user
     *
     */
    public function createAccount($userId, $password, $fname, $lname, $email = '', $privArray = '')
    {
        $serviceNode = $this->dom->createElement('AAService');
        $methodNode = $this->dom->createElement('Authenticate');
        $appNode = $this->dom->createElement($this->appId);
        $methodNode->appendChild($appNode);
        
        // Application ID node
        $tmpNode = $this->dom->createElement('AppId');
        $tmpNode->setAttribute('value',$this->appId);
        $methodNode->appendChild($tmpNode);
        
    }
    
    /**
     * Gets the user object from the A&A service tied to the given SSO token
     *
     * Enterprise A&A does support Single Sign-on. When an application uses SSO, we take them to 
     * the A&A SSO login page and when they successfully login they are returned to the calling
     * application and are passed the user's SSO token.  From there the calling application would
     * use this method to get the user object for the authenticated user
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access pulbic
     * @param string $ssoToken The SSO token tied to the user
     * @return ITE_AAClient_User A&A User object
     *
     */
    public function &getUserObject($ssoToken)
    {
        $serviceNode = $this->dom->createElement('AAService');
        $methodNode = $this->dom->createElement('GetUserObject');
        $appNode = $this->dom->createElement($this->appId);
        $methodNode->appendChild($appNode);
        
        // Application ID node
        $tmpNode = $this->dom->createElement('AppId');
        $tmpNode->setAttribute('value',$this->appId);
        $methodNode->appendChild($tmpNode);
        
        // Token ID node
        $tmpNode = $this->dom->createElement('TokenId');
        $tmpNode->setAttribute('value',$ssoToken);
        $methodNode->appendChild($tmpNode);
        
        $serviceNode->appendChild($methodNode);
        $this->dom->appendChild($serviceNode);
        
        $this->makeRequest();
        
        $userObj = $this->getUserFromResponse();
        
        $userObj->setHandler($this);
        
        return $userObj;
    }
    
    /**
     * Makes the acutal requests to the A&A Service
     *
     * Uses the cURL library to send the XML request to the A&A Service.  This method will also
     * check for any known errors we might get from the A&A service and will throw the appropriate
     * exceptions if one is encountered.
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access private
     *
     */
    private function makeRequest()
    {
        if (!function_exists('curl_init')) {
	        echo '<h1 style="font-size:30pt;color:red;font-family:arial;font-weight:bold;">The cURL libs and php functions were not found.</h1>';
	        echo '<b>The A&A libs require libcurl >= 7.12.0 and OpenSSL >= 0.9.7 31 Dec 2002</b><br/><br/>';
	        return;
	    } 
	           
        if ($this->useSSL) {
            $proto = 'https';
            if (empty($this->aaPort)) {
               $this->aaPort = 443;
            }
        } else {
            $proto = 'http';
            if (empty($this->aaPort)) {
               $this->aaPort = 80;
            }
        }
                              
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_PORT, $this->aaPort);
        curl_setopt($ch, CURLOPT_URL, sprintf('%s://%s%s', $proto, $this->aaServer, $this->aaPath));
        
        // This is need for windows OS only
        if (stristr($_ENV['OS'], 'windows')) {
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        }
        
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, sprintf('xmlInParam=%s', $this->dom->saveXML()));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $xmlResponse = curl_exec($ch) or die('error');
        curl_close ($ch); 
	    
        // Since we are reading from the socket let's get rid of any header info
        $startPos = strpos($xmlResponse,'<');
        $endPos = strrpos($xmlResponse,'>') + 1;
        $length = ($endPos - $startPos) + 1;
        
        // Load the XML response into the DOM
        //echo substr($xmlResponse, $startPos, $length); exit;
        $this->dom->loadXML(substr($xmlResponse, $startPos, $length));
        
        $this->exceptionCheck($xmlResponse);        
        
        // Now we have just the XML
        return $xmlResponse;
    }
    
    /**
     * Checks for any errors we might get from the A&A Service.  
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access private
     * @throws Exception
     *
     */
    private function exceptionCheck()
    {
        $xpath = new DOMXPath($this->dom);
        $resultList = $xpath->query('/AAService/Result');
        $tmpResult = $resultList->item(0);
        $retVal = $tmpResult->getAttribute('value');
        switch ($retVal) {
            case SUCCESSFUL_RETURN:
                return;
            case INVALID_USER_EXCEPTION:
                $msg = 'A&A Service: Invalid user ID';
                $eCode = INVALID_USER_EXCEPTION;
                break;
            case ACCOUNT_LOCKED_EXCEPTION:
                $msg = 'A&A Service: The user account is locked';
                $eCode = ACCOUNT_LOCKED_EXCEPTION;
                break;
            case CHANGE_PASSWORD_EXCEPTION:
                $msg = 'A&A Service: The user must change their password immediately';
                $eCode = CHANGE_PASSWORD_EXCEPTION;
                break;
            case INSUFFICIENT_DATA_EXCEPTION:
                $msg = 'A&A Service: The method called did not receive enough data to process the 
                    request';
                $eCode = INSUFFICIENT_DATA_EXCEPTION;
                break;
            case INVALID_NEW_PASSWORD_EXCEPTION:
                $msg = 'A&A Service: The new password given did not meet the password rules';
                $eCode = INVALID_NEW_PASSWORD_EXCEPTION;
                break;
            case ADMIN_PERMISSION_EXCEPTION:
                $msg  = 'A&A Service: The user given doe not have sufficient permission to preform
                    the requested action';
                $eCode = ADMIN_PERMISSION_EXCEPTION;
                break;
            case ADMIN_ACCOUNT_LOCKED_EXCEPTION:
                $msg = 'A&A Service: The administrator suer account is locked';
                $eCode = ADMIN_ACCOUNT_LOCKED_EXCEPTION;
                break;
            case INVALID_PRIVILEGE_EXCEPTION:
                $msg = 'A&A Service: One of the privileges given is invalid';
                $eCode = INVALID_PRIVILEGE_EXCEPTION;
                break;
            case PASSWORD_EXPIRED_EXCEPTION:
                $msg = 'A&A Service: The password for the given user has expired';
                $eCode = PASSWORD_EXPIRED_EXCEPTION;
                break;
            case ACCOUNT_DISABLED_EXCEPTION:
                $msg = 'A&A Service: The account for the given user is disabled';
                $eCode = ACCOUNT_DISABLED_EXCEPTION;
                break;
            case ADMIN_ACCOUNT_DISABLED_EXCEPTION:
                $msg = 'A&A Service: The administrator account is disabled';
                $eCode = ADMIN_ACCOUNT_DISABLED_EXCEPTION;
                break;
            case ADMIN_ACCOUNT_NOT_INITIALIZED_EXCEPTION:
                $msg = 'A&A Service: The administrator account has not been intialized yet';
                $eCode = ADMIN_ACCOUNT_NOT_INITIALIZED_EXCEPTION;
                break;
            case INVALID_TOKEN_EXCEPTION:
                $msg = 'A&A Service: An invalid Single-Sign-on token was given';
                $eCode = INVALID_TOKEN_EXCEPTION;
                break;
            case UNEXPECTED_FATAL_EXCEPTION:
                $msg = 'A&A Service: The A&A Service received an unknown fatal error';
                $eCode = UNEXPECTED_FATAL_EXCEPTION;
                break;
            default:
                $msg = 'A&A Service: The A&A Service returned an unrecognized response';
                $eCode = -1;
        }
        
        // PEAR's Exception Class.  Bringing the code in only if needed.
        require_once 'PEAR/Exception.php';
        
        // Throw the error
        throw new PEAR_Exception($msg, $eCode);
    }
    
    /**
     * Converts an A&A response into a user object
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access private
     * @return ITE_AAClient_User
     *
     */
    private function getUserFromResponse()
    {
        require_once 'User.php';
        
        $userObj = new ITE_AAClient_User();
        $xpath = new DOMXPath($this->dom);
        $tmpList = $xpath->query('/AAService/UserId');
        $tmpItem = $tmpList->item(0);
        $userObj->setUserId($tmpItem->textContent);
        $tmpList = $xpath->query('/AAService/EmpId');
        $tmpItem = $tmpList->item(0);
        $userObj->setEmpId($tmpItem->textContent);
        $tmpList = $xpath->query('/AAService/ProviderId');
        $tmpItem = $tmpList->item(0);
        $userObj->setProviderId($tmpItem->textContent);
        $tmpList = $xpath->query('/AAService/FirstName');
        $tmpItem = $tmpList->item(0);
        $userObj->setFirstName($tmpItem->textContent);
        $tmpList = $xpath->query('/AAService/LastName');
        $tmpItem = $tmpList->item(0);
        $userObj->setLastName($tmpItem->textContent);
        $tmpList = $xpath->query('/AAService/Email');
        $tmpItem = $tmpList->item(0);
        $userObj->setEmail($tmpItem->textContent);
        $tmpList = $xpath->query('/AAService/PhoneNumber');
        $tmpItem = $tmpList->item(0);
        $userObj->setPhoneNumber($tmpItem->textContent);
        
        // Get privileges
        $tmpList = $xpath->query('/AAService/PrivilegeList/Privilege');
        $userObj->setPrivileges($this->xmlPrivilegesToObjects($tmpList));
              
        return $userObj;
    }
    
    /**
     * Converts a DOM representation of privilege objects and converts them into an array
     *
     * @author Tony Bibbs <tony.bibbs@iowa.gov>
     * @access private
     * @param NodeList $domPrivs DOM representation of privileges from an XML string
     * @return array Array of privileges the user has access to
     *
     */
    private function xmlPrivilegesToObjects($domPrivs)
    {
        $userPrivs = array();
        foreach ($domPrivs as $curPriv)
        {
            $userPrivs[] = $curPriv->getAttribute('code');
        }
        return $userPrivs;        
    }
}

?>